use core::fmt::Display;

use alloc::{string::String, vec::Vec};
use serde::{Deserialize, Serialize};

#[derive(Serialize, Deserialize, Clone, Debug)]
pub struct SyslogMessage {
    pub from: String,
    pub text: Option<String>,
    pub binary: Option<SyslogBinaryMessage>,
}

#[derive(Serialize, Deserialize, Clone, Debug)]
pub struct SyslogBinaryMessage {
    pub kind: u64,
    pub data: Vec<u8>,
}

impl Display for SyslogMessage {
    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
        match (&self.text, &self.binary) {
            (None, None) => f.write_fmt(format_args!("Message from {}", self.from))?,
            (None, Some(binary)) => f.write_fmt(format_args!(
                "Message from {}: kind {}, data {:?}",
                self.from, binary.kind, binary.data
            ))?,
            (Some(text), None) => {
                f.write_fmt(format_args!("Message from {}: {}", self.from, text))?;
            }
            (Some(text), Some(binary)) => f.write_fmt(format_args!(
                "Message from {}: {} (kind {}, data {:?})",
                self.from, text, binary.kind, binary.data,
            ))?,
        };
        Ok(())
    }
}
