#[cfg(debug_assertions)]
fn print_backtrace() {
    use core::ffi::c_void;
    use unwinding::abi::{UnwindContext, UnwindReasonCode, _Unwind_Backtrace, _Unwind_GetIP};
    extern "C" fn callback(unwind_ctx: &UnwindContext<'_>, _arg: *mut c_void) -> UnwindReasonCode {
        println!("{:#x}", _Unwind_GetIP(unwind_ctx));
        UnwindReasonCode::NO_REASON
    }
    _Unwind_Backtrace(callback, core::ptr::null_mut());
}

use crate::{print, println, qemu_exit::exit_qemu, TASKING};
use core::panic::PanicInfo;

#[panic_handler]
fn panic(info: &PanicInfo<'_>) -> ! {
    print!("Kernel panic in ");
    if let Some(pid) = TASKING.current_tid().map(|x| x.pid().as_usize()) {
        print!("PID {}", pid);
    } else {
        print!("kernel init");
    }
    println!(": {info}");
    #[cfg(debug_assertions)]
    print_backtrace();
    exit_qemu();
}
